#################################################################################################
#		OUTPUT	
#################################################################################################
# this package provides an output module for the tool log2timeline.
# The package takes as an input a hash that contains all the needed information to print or output
# the timeline that has been produced by a format file
# 
# Author: Kristinn Gudjonsson
# Version : 0.6
# Date : 31/03/10
#
# Copyright 2009-2010 Kristinn Gudjonsson (kristinn ( a t ) log2timeline (d o t) net)
#
#  This file is part of log2timeline.
#
#    log2timeline is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    log2timeline is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with log2timeline.  If not, see <http://www.gnu.org/licenses/>.

package Log2t::output::sqlite;

use strict;
use DBI;
use vars qw($VERSION);
use Getopt::Long;       # read parameters
use Log2t::Common;

# version number
$VERSION = '0.6';

my $db;
#my %host;
my %source;

#       get_version
# A simple subroutine that returns the version number of the format file
#
# @return A version number
sub get_version()
{
	return $VERSION;
}

#       new
# A simple constructor of the output module. Takes care of parsing 
# parameters sent to the output module
sub new($)
{
        my $quiet;

        # read options from CMD
        @ARGV = @_;
        GetOptions(
                "quiet!"        =>\$quiet
        );
}


#       get_description
# A simple subroutine that returns a string containing a description of 
# the funcionality of the format file. This string is used when a list of
# all available format files is printed out
#
# @return A string containing a description of the format file's functionality
sub get_description()
{
	return "Output timeline into a SQLite database"; 
}

#	print_header
#
sub print_header()
{
	my $db_file;
	my $return = 1;
	my $sql;

	# get the file name
	$db_file = ::get_filename();

	# check db_file (must provide a file name)
	if( $db_file eq 'STDOUT' )
	{
		print STDERR "Unable to create database, please provide a file name to write database to (use log2timeline with the -w FILE parameter)\n";
		return 0;
	}

	# connect to a database
	$db = DBI->connect("dbi:SQLite:dbname=$db_file", "", "", { RaiseError => 1, AutoCommit => 1 }) or $return = 0;

        # %t_line {        
        #       time
        #               index
        #                       value
        #                       type
        #                       legacy
        #       desc
        #       short
        #       source
        #       sourcetype
        #       version
        #       [notes]
        #       extra
        #               [filename]
        #               [md5]
        #               [mode]
        #               [host]
        #               [user]
        #               [url]
        #               [size]
        #               [...]
        # }

	$sql ="
CREATE TABLE IF NOT EXISTS records (
	id INTEGER PRIMARY KEY AUTOINCREMENT,
	short TEXT,
	desc TEXT,
	source INTEGER,
	sourcetype TEXT,
	type INT,
	inode TEXT,
	time INT,
	hidden INT DEFAULT 0 )";

$db->do( $sql ) or $return = 0;

	# create the "extra" table
	$sql ="
CREATE TABLE IF NOT EXISTS extra (
	eid INTEGER PRIMARY KEY AUTOINCREMENT,
	rid INTEGER,
	var TEXT,
	value TEXT 
	)";

$db->do( $sql ) or $return = 0;

#	$sql ="CREATE INDEX record_atime ON records(atime)";
#	$db->do( $sql ) or $return = 0;

#	$sql ="CREATE INDEX record_mtime ON records(mtime)";
#	$db->do( $sql ) or $return = 0;

#	$sql ="CREATE INDEX record_ctime ON records(ctime)";
#	$db->do( $sql ) or $return = 0;

#	$sql ="CREATE INDEX record_crtime ON records(crtime)";
#	$db->do( $sql ) or $return = 0;

#$sql ="
#CREATE TABLE IF NOT EXISTS host (
#hid INTEGER PRIMARY KEY AUTOINCREMENT,
#name TEXT,
#timezone TEXT,
#description TEXT 
#)";
#$db->do( $sql ) or $return = 0;

$sql ="
CREATE TABLE IF NOT EXISTS source (
sid INTEGER PRIMARY KEY AUTOINCREMENT,
name TEXT,
description TEXT 
)";
$db->do( $sql ) or $return = 0;

$sql ="
CREATE TABLE IF NOT EXISTS tag (
tid INTEGER PRIMARY KEY AUTOINCREMENT,
name TEXT,
description TEXT 
)";
$db->do( $sql ) or $return = 0;

$sql ="
CREATE TABLE IF NOT EXISTS tagged (
rid INTEGER,
tid INTEGER
)";

$db->do( $sql ) or $return = 0;

	$sql ="
	CREATE TABLE IF NOT EXISTS super (
	sid INTEGER PRIMARY KEY AUTOINCREMENT,
	name TEXT,
	description TEXT
	)";
	$db->do( $sql ) or $return = 0;
	
	$sql ="
	CREATE TABLE IF NOT EXISTS super_events (
	sid INTEGER,
	rid INTEGER
	)";
	
	$db->do( $sql ) or $return = 0;

	$sql ="
	CREATE TABLE IF NOT EXISTS info (
	version TEXT,
	tool TEXT,
	ver_tool TEXT
	)";
	$db->do( $sql ) or $return = 0;

	# include information about the tool
	$sql = "INSERT INTO info (version,tool,ver_tool) VALUES( '$VERSION' , 'log2timeline', '" . Log2t::Common::get_version . '\')';
	$db->do( $sql ) or $return = 0;

	return $return;
}

sub get_footer()
{
	return 0;	# no footer
}

sub print_footer()
{
	$db->disconnect;
	return 1;
}

#      	print_line 
# A subroutine that reads a line from the access file and returns it to the
# main script
# @return A string containing one line of the log file (or a -1 if we've reached 
#       the end of the log file)
sub print_line()
{
        # content of the timestamp object t_line 
        # optional fields are marked with [] 
        # 
        # %t_line {        
        #       time
        #               index
        #                       value
        #                       type
        #                       legacy
        #       desc
        #       short
        #       source
        #       sourcetype
        #       version
        #       [notes]
        #       extra
        #               [filename]
        #               [md5]
        #               [mode]
        #               [host]
        #               [user]
        #               [url]
        #               [size]
        #               [...]
        # }

my $class = shift;
my $t_line = shift;
my $sql;
my $sth;
my $mactime;
my $sth;
my $p2;

#my %t_line = %{$ref};
if( scalar( %{$t_line} ) )
{
	# get the host ID
	#get_hid( $t_line->{'extra'}->{'host'} ) if $t_line->{'extra'}->{'host'} ne $host{'name'};

	# get the source ID
	get_sid( $t_line->{'source'} ) if $t_line->{'source'} ne $source{'name'};

	# go through each defined timestamp
	foreach( keys %{$t_line->{'time'}} )
	{
		# don't care about null values
		next unless $t_line->{'time'}->{$_}->{'value'} > 0;

		# check if we have a file (need to include MACB)
		if( lc( $t_line->{'source'}) eq 'file' )
		{              
			$mactime =  $t_line->{'time'}->{$_}->{'legacy'} & 0b0001 ? 'M' : '.';
			$mactime .= $t_line->{'time'}->{$_}->{'legacy'} & 0b0010 ? 'A' : '.';
			$mactime .= $t_line->{'time'}->{$_}->{'legacy'} & 0b0100 ? 'C' : '.';
			$mactime .= $t_line->{'time'}->{$_}->{'legacy'} & 0b1000 ? 'B' : '.';

			$p2 = '[' . $mactime . '] ';
		}
		else
		{
			$p2 = '';
		}

		# insert the record into the database
		$sql = q{INSERT INTO records (short,desc,source,sourcetype,type,inode,time,hidden) VALUES( ?,?,?,?,?,?,?,0)};
		$sth = $db->prepare( $sql );
		# execute the query
		$sth->execute(
			$db->quote( $t_line->{'short'} ),
			$db->quote( $p2 . $t_line->{'desc'} ),
			int( $source{'sid'} ),
			$db->quote( $t_line->{'sourcetype'} ),
			int( $t_line->{'time'}->{$_}->{'legacy'} ),
			$db->quote($t_line->{inode}),
			$db->quote( $t_line->{'time'}->{$_}->{'value'} )
			);

		}		

		$sql = q{SELECT id FROM records ORDER BY id DESC LIMIT 1};
		$sth = $db->prepare( $sql );
		$sth->execute();

		my @temp = $sth->fetchrow_array;
		my $sid = $temp[0];
		
		# go through the extra field (need that last sid value)
		foreach my $i ( keys %{$t_line->{'extra'}} )
		{
			$sql = q{INSERT INTO extra (rid,var,value) VALUES( ?,?,? )};
			$sth = $db->prepare( $sql );

			# execute it
			$sth->execute(
				$sid,
				$db->quote( $i ),
				$db->quote( $t_line->{'extra'}->{$i} )
			);
		}
	}
	else
	{
		print STDERR "Error. t_line not scalar\n";
		return 0;
	}

	return 1;

}

#       get_help
# A simple subroutine that returns a string containing the help 
# message for this particular format file.
# @return A string containing a help file for this format file
sub get_help()
{
	return "A output plugin that dumps all the records into a single SQLite database table,
with one extra column, hidden, indicating whether the record should be visible or not.

This database can then be read by any tool capable of reading SQLite databases.\n";

}

#sub get_hid
#{
#	my $host_name = shift;
#	my ($sth,$sql);
#	my @store;
#	# get the time zone value
#	my $tz = ::get_timezone();
#
#	# query the db
#	$sql = q{SELECT hid FROM host WHERE name = ?};
#	$sth = $db->prepare( $sql );
#	$sth->execute( $host_name );
#		
#	$host{'name'} = $host_name;
#
#	# check if we get any answer
#	if( $sth->rows eq 1 )
#	{
#		@store = $sth->fetchrow_array;
#		$host{'hid'} = $store[0];
#	}
#	else
#	{
#		# no host, create one
#		$sql = q{INSERT INTO host (name,timezone,description) VALUES( ?,?,'no description' )};
#		$sth = $db->prepare( $sql );
#		$sth->execute( $host_name, $tz );
#
#		# get the ID
#		$sql = q{SELECT hid FROM host WHERE NAME = ?};
#		$sth = $db->prepare( $sql );
#		$sth->execute( $host_name );
#
#		@store = $sth->fetchrow_array;
#		$host{'hid'} = $store[0];
#	}
#
#	return 1;
#}

sub get_sid
{
	my $source_name = shift;
	my ($sth,$sql);
	my @store;

	# query the db
	$sql = q{SELECT sid FROM source WHERE name = ?};
	$sth = $db->prepare( $sql );
	$sth->execute( $source_name );
		
	$source{'name'} = $source_name;

	# check if we get any answer
	if( $sth->rows eq 1 )
	{
		@store = $sth->fetchrow_array;
		$source{'sid'} = $store[0];
	}
	else
	{
		# no host, create one
		$sql = q{INSERT INTO source (name,description) VALUES( ?, 'no description' )};
		$sth = $db->prepare( $sql );
		$sth->execute( $source_name );

		# get the ID
		$sql = q{SELECT sid FROM source WHERE NAME = ?};
		$sth = $db->prepare( $sql );
		$sth->execute( $source_name );

		@store = $sth->fetchrow_array;
		$source{'sid'} = $store[0];
	}

	return 1;
}

1;
